import { assertEquals } from '@std/assert'
import { describe, it } from '@std/testing/bdd'
import { returnsNext, stub } from '@std/testing/mock'

import { QueryEvents } from './QueryEvents.ts'
import { ActivityResult, Address, Location } from '../adapters/jasd/types/appapi.dto.types.ts'
import { LocalTimedActivity, LocalTimedEvent, OnlineTimedActivity } from '../adapters/jasd/tests/fixtures.ts'
import { JasdGateway } from '../adapters/jasd/JasdGateway.ts'

describe('QueryEvents', () => {
  const geoApiAMock = {
    resolvePlace() {
      return Promise.resolve({ name: 'Village', city: 'Municipality', state: 'State' })
    },
  }

  it('can make sense of an empty response', async () => {
    const jasdGatewayMock = {
      fetchActivities() {
        return Promise.resolve({ events: [], totalCount: 0 })
      },
    }

    const sut = new QueryEvents(jasdGatewayMock, geoApiAMock, console)

    const result = await sut.execute({ offset: 0, limit: 1, localOnly: false })

    assertEquals(result, {
      data: [],
      totalResults: 0,
    })
  })

  it('can return a jsad event', async () => {
    const jasdGatewayMock = {
      fetchActivities(): Promise<{ events: ActivityResult[]; totalCount: number }> {
        return Promise.resolve({
          events: [LocalTimedEvent as ActivityResult],
          totalCount: 1,
        })
      },
    }

    const sut = new QueryEvents(jasdGatewayMock, geoApiAMock, console)

    const result = await sut.execute({ offset: 0, limit: 1, localOnly: false })

    assertEquals(result, {
      data: [
        {
          id: '4610',
          title: 'RealTimedEvent',
          organisationName: 'Allerweltskino e.V.',
          startDate: '2025-03-26T00:00:00+01:00',
          endDate: '2025-03-26T00:00:00+01:00',
          location: {
            city: 'Köln',
            street: 'Bahnhofstr.',
            streetNo: '41',
            zipCode: '50674',
          },
          isRemote: false,
          imageUrl: 'https://gemeinschaftswerk-nachhaltigkeit.de/app/api/v1/files/activities/act5.jpg',
          eventDetails: {
            description:
              '<p>Jeden Dienstag (bis auf eine Sommer- und Winterpause) zeigt der Verein Allerweltskino e.V. im Kölner Filmtheater Off Broadway einen Nachhaltigkeitsfilm. Das Programm ist im Internet abrufbar (siehe Kasten rechts).</p>',
            externalLink: 'https://www.allerweltskino.de',
            meetingLink: 'https://www.allerweltskino.de',
          },
        },
      ],
      totalResults: 1,
    })
  })

  describe('when requesting 5 events', () => {
    it('it puts a events in the users city first in the response', async () => {
      const activitiesForCityResponse = {
        events: [
          {
            name: 'a',
            description: 'description',
            resultType: 'ACTIVITY',
            activity: LocalTimedActivity,
            location: aLocation({ address: anAddress({ city: 'Municipality' }) }),
          },
          {
            name: 'b',
            description: 'description',
            resultType: 'ACTIVITY',
            activity: LocalTimedActivity,
            location: aLocation({ address: anAddress({ city: 'Municipality' }) }),
          },
        ],
        totalCount: 2,
      }

      const activitiesForStateResponse = {
        events: [
          {
            name: 'c',
            description: 'description',
            resultType: 'ACTIVITY',
            activity: LocalTimedActivity,
            location: aLocation({ address: anAddress({ state: 'State' }) }),
          },
        ],
        totalCount: 1,
      }
      const activitiesUnfilteredResponse = {
        events: [
          {
            name: 'd',
            description: 'description',
            resultType: 'ACTIVITY',
            activity: LocalTimedActivity,
            location: LocalTimedActivity.location,
          },
          {
            name: 'e',
            description: 'description',
            resultType: 'ACTIVITY',
            activity: OnlineTimedActivity,
            location: OnlineTimedActivity.location,
          },
        ],
        totalCount: 2,
      }

      const fetchActivitiesSpy = stub(
        {} as JasdGateway,
        'fetchActivities',
        returnsNext([
          Promise.resolve(activitiesForCityResponse),
          Promise.resolve(activitiesForStateResponse),
          Promise.resolve(activitiesUnfilteredResponse),
        ]),
      )

      const sut = new QueryEvents(
        {
          fetchActivities: fetchActivitiesSpy,
        },
        geoApiAMock,
        console,
      )

      const result = await sut.execute({
        limit: 5,
        offset: 0,
        localOnly: false,
        geolocationId: 'someGeolocationId',
      })

      assertEquals(result.totalResults, 5)
      assertEquals(result.data[0].title, 'a')
      assertEquals(result.data[1].title, 'b')
      assertEquals(result.data[2].title, 'c')
      assertEquals(result.data[3].title, 'd')
      assertEquals(result.data[4].title, 'e')
    })
  })
})

// todo gregor: test remote events in the location-based set are ignored

const aLocation = (override?: Partial<Location>): Location => {
  const fallback: Location = {
    address: anAddress(),
    coordinate: {
      coordinates: [0, 0],
      type: 'Point',
    },
    online: false,
    privateLocation: false,
    url: '',
  }

  return { ...fallback, ...override }
}

const anAddress = (override?: Partial<Address>): Address => {
  const fallback = {
    city: '',
    country: null,
    name: '',
    state: null,
    street: '',
    streetNo: '',
    supplement: null,
    zipCode: '',
  }
  return { ...fallback, ...override }
}
